@@ -2,14 +2,17 @@ use Test;
 
 BEGIN { plan tests => 2 };
 
+# Curses::UI
 use Curses::UI;
+
+# The base classes
+use Curses::UI::Widget;
+use Curses::UI::Container;
+
+# The widget classes
 use Curses::UI::Buttons;
 use Curses::UI::CheckBox;
 use Curses::UI::Common;
-use Curses::UI::Container;
-use Curses::UI::Dialog;
-use Curses::UI::ErrorDialog;
-use Curses::UI::FileBrowser;
 use Curses::UI::Label;
 use Curses::UI::ListBox;
 use Curses::UI::MenuBar;
@@ -22,9 +25,15 @@ use Curses::UI::Searchable;
 use Curses::UI::TextEditor;
 use Curses::UI::TextEntry;
 use Curses::UI::TextViewer;
-use Curses::UI::Widget;
 use Curses::UI::Window;
 
+# The dialogs
+use Curses::UI::Dialog::Basic;
+use Curses::UI::Dialog::Error;
+use Curses::UI::Dialog::Status;
+use Curses::UI::Dialog::FileBrowser;
+use Curses::UI::Dialog::Progress;
+
 ok(1);
 
 system "examples/basic_test";
